#include <raylib.h>
#include <raymath.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <threads.h>

#include "faces.h"
#include "chunk.h"
#include "vector.h" 
#define STB_PERLIN_IMPLEMENTATION
#include "stb_perlin.h"

Mesh gen_mesh(Chunk chunk) {
	Mesh mesh = { 0 };
	mesh.vertices = malloc(0);
	mesh.vertexCount = 0;
	mesh.texcoords = malloc(0);

	for(int x = 0; x < WIDTH; x++) {
		for (int y = 0; y < HEIGHT; y++) {
			for (int z = 0; z < WIDTH; z++) {
				vector3 pos = vec3(x, y, z);
				int block_id = get_block_at(chunk, pos);
				if (block_id == 0) continue;
				
				if (get_block_at(chunk, vec3(x, y - 1, z)) == 0)
					add_face(&mesh, bottom, bottom_tex, block_id, pos);

				if (get_block_at(chunk, vec3(x, y + 1, z)) == 0)
					add_face(&mesh, top, top_tex, block_id == 1 ? 4 : block_id, pos);

				if (get_block_at(chunk, vec3(x - 1, y, z)) == 0)
					add_face(&mesh, left, left_tex, block_id, pos);

				if (get_block_at(chunk, vec3(x + 1, y, z)) == 0)
					add_face(&mesh, right, right_tex, block_id, pos);

				if (get_block_at(chunk, vec3(x, y, z - 1)) == 0)
					add_face(&mesh, back, back_tex, block_id, pos);

				if (get_block_at(chunk, vec3(x, y, z + 1)) == 0)
					add_face(&mesh, front, front_tex, block_id, pos);
			}
		}
	}

	UploadMesh(&mesh, false);

	printf("mesh generated, %d verts, %d tris\n", mesh.vertexCount, mesh.triangleCount);

	return mesh;
}

int main() {
	InitWindow(800, 600, "samcraft");

	//SetTargetFPS(60);

	Chunk* chunks = calloc(2, sizeof(Chunk));
	Mesh* chunk_meshes = calloc(2, sizeof(Mesh));

	for (int i = 0; i < 2; i++) {
		for (int x = 0; x < WIDTH; x++) {
			for (int z = 0; z < WIDTH; z++) {
				int height = (90 + stb_perlin_fbm_noise3(((i * WIDTH) + x) / 40.0f, x / 40.0f, z / 40.0f, 2.0f, 0.5f, 6) * 10.0f);
				for (int y = 0; y < height; y++) {
					int id = 2;
					if (height - y == 1) id = 1;
					if (height - y == 2) id = 3;
					chunks[i][coord_to_index((vector3){x, y, z})] = id;
				}
			}
		}

		Mesh mesh = gen_mesh(chunks[i]);
		chunk_meshes[i] = mesh;
	}

	Texture atlas = LoadTexture("atlas.png");
	Material mat = LoadMaterialDefault();
	mat.maps[MATERIAL_MAP_DIFFUSE].texture = atlas;

	Camera camera = { 0 };
	camera.position = (Vector3){ 10.0f, 10.0f, 10.0f };
	camera.target = (Vector3){ 0.0f, 0.0f, 0.0f };
	camera.up = (Vector3){ 0.0f, 1.0f, 0.0f };
	camera.projection = CAMERA_PERSPECTIVE;
	camera.fovy = 70.0f;

	while (!WindowShouldClose()) {
		UpdateCamera(&camera, CAMERA_FREE);

		DisableCursor();

		BeginDrawing();
		{
			ClearBackground(SKYBLUE);

			BeginMode3D(camera);
			{
				for (int i = 0; i < 2; i++) {
					DrawMesh(chunk_meshes[i], mat, MatrixTranslate(i * WIDTH, 0.0f, 0.0f));
				}
				DrawGrid(10, 10);
			}
			EndMode3D();

			DrawFPS(10, 10);
		}
		EndDrawing();
	}

	CloseWindow();

	return 0;
}
